unit tcpt;
{
  Kokeilu TCP/IP -yhteyden kyttmisest.  Sama ohjelma toimii
  sek palvelimena ett asiakkaana. Voi toimia mys palvelimena
  itselleen!  Oletuksena ohjelma kytkeytyy palvelimeksi, eli
  kuuntelee porttia 1024.

  13.12.1997 / Vesa Lappalainen

  Huomio!  TTCP-komponentit eivt toimi, jollei niiden
           state -ominaisuuteen viitata ennen kytt!

           Jos TCPCli yhteys katkaistaan, ei saada uutta yhteytt,
           jollei ennen uutta yhteytt aseteta TCPCli.LocalPort := 0;

  Komponentit:
    TCPCli  - yhteys asiakkaana olemista varten.  Tm ottaa yhteyden
              koneen nimell tai IP-osoitteella johonkin olemassa
              olevaan palvelimeen sen portin tunnuksella.
              Jos yhteys saadaan, tulee event OnConnect.
              Dataa lhetetn metodilla SendData ja vastaanotetaan
              tapahtumassa OnDataArrival.

    TCPServ - tm toimii palvelimena.  Palvelin aktivoidaan metodilla
               Listen.  Tllin tytyy LocalPort olla asettu halutuksi

    TCPAcp  - tt kytten hyvksymn TCPServilt pyydetty yhteys.
              Tm hoitaa varsinaisen liikenteen lhettmll (SendData)
              ja vastaanottamalla (event OnDataArrival) dataa.


}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, OleCtrls, isp3, ActiveX;

type
  TFormTCPTest = class(TForm)
    TCPServ: TTCP;
    TCPAcp: TTCP;
    TCPCli: TTCP;
    ButtonCliConnect: TButton;
    LabelServ: TLabel;
    LabelCli: TLabel;
    ButtonCliSend: TButton;
    ButtonListen: TButton;
    ButtonClose: TButton;
    LabelAcpData: TLabel;
    ButtonAcpSend: TButton;
    LabelCliData: TLabel;
    LabelAcp: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    LabelError: TLabel;
    EditHost: TEdit;
    Label4: TLabel;
    Label5: TLabel;
    EditCliPort: TEdit;
    EditServPort: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure TCPServConnectionRequest(Sender: TObject;
      requestID: Integer);
    procedure TCPAcpDataArrival(Sender: TObject; bytesTotal: Integer);
    procedure TCPAcpClose(Sender: TObject);
    procedure ButtonCliConnectClick(Sender: TObject);
    procedure TCPCliDataArrival(Sender: TObject; bytesTotal: Integer);
    procedure ButtonCliSendClick(Sender: TObject);
    procedure TCPCliConnect(Sender: TObject);
    procedure TCPCliError(Sender: TObject; Number: Smallint;
      var Description: WideString; Scode: Integer; const Source,
      HelpFile: WideString; HelpContext: Integer;
      var CancelDisplay: WordBool);
    procedure ButtonListenClick(Sender: TObject);
    procedure ButtonCloseClick(Sender: TObject);
    procedure ButtonAcpSendClick(Sender: TObject);
    procedure TCPServClose(Sender: TObject);
    procedure TCPCliClose(Sender: TObject);
  private
    { Private declarations }
//    TCPCli: TTCP;
    ServSend : integer;
    CliSend  : integer;
    procedure CloseAll;
    procedure DoListen;
    function  CloseTCPCli : boolean;
  public
    { Public declarations }
  end;

var
  FormTCPTest: TFormTCPTest;

implementation

{$R *.DFM}

//------------------------------------------------------------------------------
function IncStr(var i:integer):string;
begin
  inc(i);
  Result := IntToStr(i);
end;

//------------------------------------------------------------------------------
procedure TFormTCPTest.FormCreate(Sender: TObject);
begin
  DoListen;
end;

//------------------------------------------------------------------------------
procedure TFormTCPTest.TCPServConnectionRequest(Sender: TObject;
  requestID: Integer);
begin
  TCPServ.Close;                  // Ei hyvksyt en toista yhteytt
  TCPAcp.Accept(requestID);       // Yhteyden hoitaa jatkossa TCPAcp
  LabelServ.Caption := 'Connect req: ' + TCPServ.RemoteHostIP;
  ButtonAcpSend.Enabled := true;  // Lhetysnappula aktiiviseksi
end;

//------------------------------------------------------------------------------
procedure TFormTCPTest.TCPCliDataArrival(Sender: TObject; bytesTotal: Integer);
var data : OleVariant;
begin
  TCPCli.GetData(data, VT_BSTR,bytesTotal);
  LabelCliData.Caption := data;
end;

procedure TFormTCPTest.TCPAcpDataArrival(Sender: TObject; bytesTotal: Integer);
var data : OleVariant;
begin
  TCPAcp.GetData(data, VT_BSTR,bytesTotal);
  LabelAcpData.Caption := Data;
end;

//------------------------------------------------------------------------------
procedure TFormTCPTest.TCPCliError(Sender: TObject; Number: Smallint;
  var Description: WideString; Scode: Integer; const Source,
  HelpFile: WideString; HelpContext: Integer; var CancelDisplay: WordBool);
begin
  LabelError.Caption := (Sender as TTCP).Name +
                        ' Error: ' + Description + ' ' + source;
end;


//------------------------------------------------------------------------------
procedure TFormTCPTest.TCPCliConnect(Sender: TObject);
begin
  ButtonCliSend.Enabled := True;
  LabelCli.Caption := 'Connected to ' + TCPCli.RemoteHostIP;
end;

//------------------------------------------------------------------------------
procedure TFormTCPTest.ButtonCliConnectClick(Sender: TObject);
begin
  CloseTCPCli;
  TCPCli.LocalPort := 0;
  TCPCli.Connect(EditHost.Text,EditCliPort.Text);
  LabelCli.Caption := 'Waiting connect';
end;

//------------------------------------------------------------------------------
procedure TFormTCPTest.ButtonListenClick(Sender: TObject);
begin
  DoListen;
end;

//------------------------------------------------------------------------------
procedure TFormTCPTest.ButtonCloseClick(Sender: TObject);
begin
  CloseAll;
end;

//------------------------------------------------------------------------------
procedure TFormTCPTest.ButtonCliSendClick(Sender: TObject);
begin
  TCPCli.SendData('FromClient: '+IncStr(CliSend));
end;

procedure TFormTCPTest.ButtonAcpSendClick(Sender: TObject);
begin
  TCPAcp.SendData('FromServer: '+IncStr(ServSend));
end;

//------------------------------------------------------------------------------
procedure TFormTCPTest.TCPCliClose(Sender: TObject);
begin
  LabelCli.Caption := 'Cli Close';
  ButtonCliSend.Enabled := false;
end;

procedure TFormTCPTest.TCPServClose(Sender: TObject);
begin
  LabelServ.Caption := 'Serv Close';
end;

procedure TFormTCPTest.TCPAcpClose(Sender: TObject);
begin
  LabelAcp.Caption := 'Acp Close';
  LabelServ.Caption := '';
  ButtonAcpSend.Enabled := false;
end;

//------------------------------------------------------------------------------
procedure TFormTCPTest.CloseAll;
begin
  CloseTCPCli;
  if TCPServ.State <> sckClosed then TCPServ.Close;
  if TCPAcp.State <> sckClosed then  TCPAcp.Close;
  ButtonAcpSend.Enabled := false;

  LabelServ.Caption := '';
  LabelAcp.Caption := '';
  LabelCliData.Caption := '';
  LabelAcpData.Caption := '';
  LabelError.Caption := '';
end;

//------------------------------------------------------------------------------
procedure TFormTCPTest.DoListen;
begin
  CloseAll;
  TCPServ.LocalPort := StrToIntDef(EditServPort.Text,1024);
  TCPServ.Listen;
  LabelServ.Caption := 'Listen';
end;

//------------------------------------------------------------------------------
function TFormTCPTest.CloseTCPCli : boolean;
begin
  LabelCli.Caption := '';
  ButtonCliSend.Enabled := false;
  if TCPCli.State <> sckClosed then
    TCPCli.Close;
  Result := true;
end;


end.
